﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Threading; 

namespace LACS_WF
{
    public partial class Form1 : Form
    {
        LARunClass m_LARun;
        int m_id, m_iCurSize;
		UInt32[] m_lpiWave;
        string csModelName;
		string csErrorString;
		bool gfTrMode;
		LARunClass.LA_HW_MODE m_giHwMode;
        LARunClass.TRIG m_tr = new LARunClass.TRIG();
        DispatcherTimer MainTimerClock;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            m_LARun = new LARunClass();
			m_giHwMode = LARunClass.LA_HW_MODE.HW_200M_9CH;
			gfTrMode = false;

			byte[] szBuf = System.Text.Encoding.ASCII.GetBytes("12345" + "\0");
			int iSize = szBuf.Count();
			int c = 0;
        }

        private void btn_GetHW_Click(object sender, EventArgs e)
        {
            int i;

			if (m_LARun.ulaSDKInit())
			{
				m_LARun.ulaSDKThreshold(0, 1600);			// 1.6V (TTL)
				m_LARun.ulaSDKThreshold(1, 1600);			// 1.6V (TTL)
				m_LARun.ulaSDKThreshold(2, 1600);			// 1.6V (TTL)
				m_LARun.ulaSDKThreshold(3, 1600);			// 1.6V (TTL)

				m_id = m_LARun.ulaSDKGetLaID();

				for (i = 0; LARunClass.m_InfoLst[i].wID != 0; i++)
				{
					if (LARunClass.m_InfoLst[i].wID == m_id)
						break;
				}

				if (LARunClass.m_InfoLst[i].wID != 0)
					csModelName = string.Format("{0}: HW Mode = {1}", LARunClass.m_InfoLst[i].pName, m_giHwMode.ToString());
				else
					csModelName = "Unknown";

				edLAModel.Text = csModelName;

				btn_Capture.Enabled = true;
				btn_GetHW.Enabled = false;
				lbStatusDisplay.Text = "H/W Initialized";
			}
			else
			{
				ShowErrorCode();
			}
        }

		private void SetTrigger()
		{
			int iTrigChannel = 1;
			int iTrigLevel = 0;
			int iTrigCondition = LARunClass.LA_TRIG_CHANGE;
			int iTrigContinue = LARunClass.TR_CONTINUE;
			if (!m_LARun.ulaSDKClearTrigger(ref m_tr))	//clear the trigger setting to free run
			{
				ShowErrorCode();
				return;
			}
			if (!m_LARun.ulaSDKSetChTrigger(ref m_tr, iTrigLevel, iTrigChannel, iTrigCondition, iTrigContinue))	//set trigger for level 0
			{
				ShowErrorCode();
				return;
			}
			//if (!m_LARun.ulaSDKSetChTrigger(ref m_tr, iTrigLevel2, iTrigChannel2, iTrigCondition2, iTrigContinue2))	//set trigger for level 1 if need
			//{
			//    ShowErrorCode();
			//    return;
			//}
		}

        private void btn_Capture_Click(object sender, EventArgs e)
        {
            btn_Capture.Enabled = false;
            btn_Stop.Enabled = true;

			//Set trigger condition to single level trigger
			SetTrigger();
			
            m_tr.iDelay = 0;						// for LA2000P
            m_tr.iExtClk = 1;						// 1: Use Internal Clock
            m_tr.iFlag = LARunClass.TR_PRETRIG;
			m_tr.iFreq = (uint)(200E6);
			m_tr.iFreqHi = (int)((UInt64)200E6 >> 32);
            m_tr.iPassCount = 0;					// No Pass Count
            m_tr.iTrPos = 50;						// Trigger signal located at point 50
            m_tr.iWidth = 0;						// for LA2000P

			if (!this.ckTransMode.Checked)	//Standard conventional storage
				m_giHwMode = LARunClass.LA_HW_MODE.HW_200M_9CH;
			else//Transitional storage 
				m_giHwMode = LARunClass.LA_HW_MODE.HW_200M_32CH_TR;
			gfTrMode = this.ckTransMode.Checked;

			//Select HW mode
			if (!m_LARun.ulaSDKSetHwInfo((int)LARunClass.LA_HW_SETTING.SET_TL_MODE, (int)m_giHwMode))
			{
				ShowErrorCode();
				return;
			}

			//Select Channel mask
			byte[] iChannelMask = new byte[32];
			Array.Clear(iChannelMask, 0, 32);
			iChannelMask[0] = 0x08 | 0x10; //Mask channel 3, 4, 5, ignore all value change in these channels
			if (!m_LARun.ulaSDKSetChannelMask(iChannelMask, 32))
			{
				ShowErrorCode();
				return;
			}

			//Set H/W filter
			int iFilterTime_ns = 20;	//20ns
			Int64 i64ChFilterOnOff = 0;
			i64ChFilterOnOff |= 0x01 << 0;	//Set CH0 Filter on
			i64ChFilterOnOff |= 0x01 << 5;	//Set CH5 Filter on
			if (!m_LARun.ulaSDKSetHwFilter(i64ChFilterOnOff, iFilterTime_ns))
			{
				ShowErrorCode();
				return;
			}

			if (!gfTrMode)	//memory setting not required in transitional storage mode
			{
				// Just to know the LA depth pre channel
				int iSamplesPerCh = m_LARun.ulaSDKGetMaxSamplesNum();

				// I just need 2000 samples
				if (!m_LARun.ulaSDKSetSamplesNum(2000))  // Set 2000 samples
				{
					ShowErrorCode();
					return;
				}
				// Get the buffer size(in bytes) for 2000 samples
				m_iCurSize = m_LARun.ulaSDKGetBufferSizeInBytes();
				m_lpiWave = new UInt32[m_iCurSize];
			}
			
			//Set Trigger position
			if (!gfTrMode)
				m_tr.iTrPos = 1000 / 2000;						// Trigger signal located at 50%, 2000samples -> 1000
			else
			{
				int iSamplesPerCh = m_LARun.ulaSDKGetMaxSamplesNum();
				m_tr.iTrPos = iSamplesPerCh / 2;
			}

			if (!m_LARun.ulaSDKCapture(ref m_tr))
			{
				ShowErrorCode();
				return;
			}

            MainTimerClock = new DispatcherTimer();
            //Creates a timerClock and enables it
            MainTimerClock.Interval = new TimeSpan(100);
            MainTimerClock.IsEnabled = true;
            MainTimerClock.Tick += new EventHandler(MainTimerClockProc);
			lbStatusDisplay.Text = "Capturing...";
        }

        public void MainTimerClockProc(object sender, EventArgs e)
        {
			//Check Trigger ready
			if (m_LARun.ulaSDKIsTriggerReady() != false)
				lbStatusDisplay.Text = "Capturing...Triggered";
			else
				lbStatusDisplay.Text = "Capturing...";
            // Wait Data ready
			if (m_LARun.ulaSDKIsCaptureReady() != false)
            {
                btn_Capture.Enabled = true;
                btn_Stop.Enabled = false;
                MainTimerClock.Stop();
				lbStatusDisplay.Text = "Capture Finished";

                // When data ready then read it.
				if (!gfTrMode)
				{
					if (!m_LARun.ulaSDKGetBusData(7, 0, m_lpiWave, m_iCurSize, 0))
					{
						ShowErrorCode();
						return;
					}
				}
				else
				{//Collect data to a text file, no waveform drawing here
					ReadTrWaveAndWriteFile();				
				}
                Invalidate();
            }
        }

		private void ReadTrWaveAndWriteFile()
		{
			//Debug used Environment setting
			const string strFilePath = "D:\\LASDK_TransText.txt";
			const int iMaxLine = 200;
			///////////////////////////////////

			string strTimestamp;
			string strData;
			string strText;
			const Int64 i64EndSign = -1;
			const int iDataSize = 8;	//Max 64Channel
			Int64 i64TimestampHead = 0;
			Int64 i64TimestampTrig = 0;
			Int64 i64Timestamp = 0;
			int iLine = 0;
			byte[] lpbWav = new byte[iDataSize];
			double dbSampleInterval_ns = 1E9 / m_tr.iFreq;

			System.IO.StreamWriter swFile = System.IO.File.CreateText(strFilePath);
			swFile.Write("Timestamp, CH  0  8  16  32\r\n");
			if (m_LARun.ulaSDKGetTransStoreInfo(ref i64TimestampHead, ref i64TimestampTrig))
			{
				i64Timestamp = i64TimestampHead;
				while (i64Timestamp != i64EndSign)
				{
					strTimestamp = ((i64Timestamp - i64TimestampHead) * dbSampleInterval_ns).ToString() + "ns";
					strData = "";
					for (int i = 0; i < iDataSize; i++)
						strData += string.Format("{0:X2}", lpbWav[i]) + " ";

					strText = strTimestamp + "," + strData + "\r\n";
					swFile.Write(strText);
					iLine++;

					if (iLine > iMaxLine)	//debug max line check
						break;

					i64Timestamp = m_LARun.ulaSDKGetTrWfm(i64Timestamp, lpbWav);
				}
				swFile.Close();
			}

		}

        private void btn_Stop_Click(object sender, EventArgs e)
        {
            MainTimerClock.Stop();
			if (!m_LARun.ulaSDKStopCapture())
			{
				ShowErrorCode();
				return;
			}
			if (!gfTrMode)
			{
				if (!m_LARun.ulaSDKGetBusData(7, 0, m_lpiWave, m_iCurSize, 0))
				{
					ShowErrorCode();
					return;
				}
			}
			else
			{//Collect data to a text file, no waveform drawing here
				ReadTrWaveAndWriteFile();
			}
			btn_Capture.Enabled = true;
			btn_Stop.Enabled = false;
			//if (!m_LARun.ulaSDKGetBusData(7, 0, m_lpiWave, m_iCurSize, 0)) // Stop capture       
			//{
			//    ShowErrorCode();
			//    return;
			//}
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
			if (m_lpiWave == null)
                return;
            int iCh;       
            int[] iLabYH = new int[8];
            int[] iLabYL = new int[8];
            int[] iLabX = new int[8];
            Label[] LabChannel = new Label[]
            {
                Lab_Ch0, Lab_Ch1, Lab_Ch2, Lab_Ch3,
                Lab_Ch4, Lab_Ch5, Lab_Ch6, Lab_Ch7,
            };
            
            // Get Draw position 
            Point DrawPos = Wave_Canvas.PointToScreen(new Point(0, 0));
            DrawPos = PointToClient(DrawPos);
            for (iCh = 0; iCh < 8; iCh++)
            {
                Point LabPos = LabChannel[iCh].PointToScreen(new Point(0, 0));
                LabPos = PointToClient(LabPos);  
                                     
                iLabYH[iCh] = (int)LabPos.Y + 3;
                iLabYL[iCh] = (int)(LabPos.Y + LabChannel[iCh].Height) - 3;
                iLabX[iCh] = (int)(DrawPos.X + 3);       
            }
                       
            int iIndex, iShowWidth = (int)Wave_Canvas.Width-3;                       
            Point p1 = new Point();           

            for (iCh = 0; iCh < 8; iCh++)
            {
                int iDrawX;
                UInt64 ui64Temp, ui64Temp1, ui64ChMask = (UInt64)1 << iCh;

                List<Point> ptPoly = new List<Point>();
                // 1st point
                p1.X = iLabX[iCh];
                ui64Temp = m_lpiWave[0] & ui64ChMask;
                if (ui64Temp != 0)
                    p1.Y = iLabYH[iCh];
                else
                    p1.Y = iLabYL[iCh];
                
                ptPoly.Add(new Point(p1.X, p1.Y));

                for (iIndex = 1, iDrawX = iLabX[iCh] + 1; iIndex < iShowWidth; iIndex++, iDrawX++)
                {
					ui64Temp = m_lpiWave[iIndex - 1] & ui64ChMask;
                    ui64Temp1 = m_lpiWave[iIndex] & ui64ChMask;
                    if ((ui64Temp & ui64Temp1) == ui64ChMask)
                    {
                        p1.X = iDrawX;
                        p1.Y = iLabYH[iCh];
                        ptPoly.Add(new Point(p1.X, p1.Y));
                    }
                    else if (ui64Temp1 == ui64ChMask)
                    {
                        p1.X = iDrawX - 1;
                        p1.Y = iLabYH[iCh];
                        ptPoly.Add(new Point(p1.X, p1.Y));                       
                        p1.X = iDrawX;
                        p1.Y = iLabYH[iCh];
                        ptPoly.Add(new Point(p1.X, p1.Y));
                    }
                    else if (ui64Temp == ui64ChMask)
                    {
                        p1.X = iDrawX;
                        p1.Y = iLabYH[iCh];
                        ptPoly.Add(new Point(p1.X, p1.Y));
                        p1.X = iDrawX;
                        p1.Y = iLabYL[iCh];
                        ptPoly.Add(new Point(p1.X, p1.Y));
                    }
                    else
                    {
                        p1.X = iDrawX;
                        p1.Y = iLabYL[iCh];
                        ptPoly.Add(new Point(p1.X, p1.Y));
                    }
                }

                Pen pen = new Pen(Color.Black, 1);
           
                Point[] ptDrawPoly = new Point[ptPoly.Count];
                for (int i = 0; i < ptPoly.Count; i++)
                {
                    ptDrawPoly[i] = ptPoly[i];
                }

                e.Graphics.DrawLines(pen, ptDrawPoly);                              
            }
        }

		private void ShowErrorCode()
		{
			int iErrorCode = m_LARun.ulaSDKGetLastError();
			csErrorString = "ErrorCode = " + ((LA_ErrorCode.ErrorCode)iErrorCode).ToString();
			MessageBox.Show(csErrorString);
		}

		private void button1_Click(object sender, EventArgs e)
		{
			if (!m_LARun.ulaSDKSaveAsLawFile("D:\\LaSDKCsharp.law"))
				ShowErrorCode();
		}

		private void bnSaveWithTemplate_Click(object sender, EventArgs e)
		{
			if (!m_LARun.ulaSDKSaveAsLawFileWithTemplate("D:\\LaSDKCsharp_Ex.law", "SPI_Test.law"))
				ShowErrorCode();
		}
    }
}
